function [a,r] = linefit(x)

%LINEFIT  Fits a straight line to points in a plane minimizing sum of Euclidean distances.
%         [a,r] = linefit(x), where
%           a ... Size (1,3). Line parameters.
%           x ... Size (2,N). Points in a plane.
%           r ... Size (1,N). Residuals = Euclidean distances from the fitted line.
%         Fitting is done by minimizing 
%           min{ sum(|a*[x; ones]|) }
%         subject to a(1)^2+a(2)^2==1.

%       Author       : (c) T. Werner and T. Pajdla, {werner,pajdla}@cmp.felk.cvut.cz
%                      07/09/98
%                      Center for Machine Perception, Czech Technical University, Prague
%       Documentation: ftp://cmp.felk.cvut.cz/pub/cmp/articles/werner/tr97-138.ps.gz     
%       Language     : Matlab 5.1, (c) MathWorks                         
%       Last change  : $Id: linefit.m,v 1.1 1998/09/07 15:54:15 Matlab Exp $
%       Status       : Ready ($Source: /home.others/Matlab/Matlab/radial/RCS/linefit.m,v $)


%%
 % The constrained fit is done according to: 
 % Faugeras: Computer Vision, A Geometric Approach, page 623.
 % If matrix D = x', and C = a(3)*ones(N,1), E = D'*(eye(N)-C*inv(C'*C)*C')*D,
 % then the solution a(1:2) is the eigenvector of E'*E corresponding to minimal
 % eigenvalue, or, equivalently, the last column of d, [s,v,d]=svd(E).
 % The missing value a(3) can be easily obtained from min{ |a(3)*ones(N,1)+D*a(1:2)| }
 % as the mean, a(3)=-sum(D*a(1:2))/N.
 %%

N = size(x,2);

%  A = x * (eye(N)-ones(N)/N) * x';
%  A = x*x' - x*ones(N)/N*x';

sx = sum(x')';
[s,v,d] = svd(x*x'-sx*sx'/N);
a = d(:,size(d,2))';
a = [a -sum(a*x)/N];
r = abs(a*[x; ones(1,N)]);

return

% example
x=[0 0; 0 2; 1 0; 1 2]'; [a,r] = linefit(x); clf, plot(x(1,:),x(2,:),'x'), hold on, pline(a); hold off, axis('equal')
