function [p,H] = rdestim(method,p0,ud,L_uw,SWAP_DISTFCN)
%rdestim  Estimating radial lens distortion.
%
%       p = rdestim('lines',p0,ud,L) estimates distortion parameters from lines in scene.
%       Parameters:
%         p0 ... The initial estimate of radial lens distortion parameters.
%                p0 = [u0; r0; k], see R2RDIST for details.
%         ud ... Size (2,?). Considered distorted image points of which some are colinear.
%         L ... Size (?,N), sparse. Non-zero entries in a column are indices of points
%               ui which are colinear in the scene. There are N lists of colinear points.
%         p ... Size=size(p0). Estimate of the radial lens distorion parameters.
%       Algorithm:
%         The radial lens distortion parameters p are estimated using the knowledge that
%         some points from ud are colinear in the world. Due to the lens distortion, they 
%         are not colinear in the image.
%           The algorithm estimates the parameters p by minimizing residuals of fitting 
%         lines to colinear points from ud, after they were "undistorted" by inverse 
%         distortion function of p.
%       Notes:
%         Optimization runs always with fixed r0 (=p(3)).
%
%       [p,H] = rdestim('homo',p0,ud,uw)
%       Parameters:
%         p0, p ... As above.
%         ud ... Size (2,M). Projections of coplanar scene points uw to image plane 
%                by a camera with radial lens distortion.
%         uw ... Size (2,M). Coplanar scene points known up to a 2D-to-2D homography.
%         H ... Size (3,3). Homography estimate. It is approx. 
%                 ud = UDIST2U(p,p2e(H*e2p(uw))).
%       Algorithm:
%         Using image points ud (projected from the world by a perspective camera 
%         with radial distortion) and corresponding world points uw known up to 
%         a 2D-to-2D homography H (the world points are coplanar), the function 
%         estimates homography H and camera radial distortion parameters p. The 
%         estimation is done by gradient descent. The initial value of H is estimated 
%         automatically from ud and uw, the initial value of p must be given.
%       Notes:
%         Optimization runs always with fixed r0 (=p(3)).
%
%       See also u2udist, udist2u.

%       Author       : (c) T. Werner and T. Pajdla, {werner,pajdla}@cmp.felk.cvut.cz
%                      07/09/98
%                      Center for Machine Perception, Czech Technical University, Prague
%       Documentation: ftp://cmp.felk.cvut.cz/pub/cmp/articles/werner/tr97-138.ps.gz     
%       Language     : Matlab 5.1, (c) MathWorks                         
%       Last change  : $Id: rdestim.m,v 1.1 1998/09/07 15:54:15 Matlab Exp $
%       Status       : Ready ($Source: /home.others/Matlab/Matlab/radial/RCS/rdestim.m,v $)


if exist('SWAP_DISTFCN') == 0,
  SWAP_DISTFCN = 0;
end


%%
 % Normalize ud, p0 to be roughly between -1 and 1.
 % The reason is that the finite differences in gradient descent work correctly. 
 %%
Aud = normu(ud);
ud = p2e(Aud*e2p(ud));
p0 = psimi2p(p0,Aud);

%%
 % Set options of the gradient descent
 %%
global iter
global LSTSQOPT
LSTSQOPT = 'lstsq.log';
[dummy,dummy,OPTIONS] = lsint([],[],[]);
  OPTIONS( 1) = 1;        % Display parameter (Default:0). 1 displays some results
  OPTIONS( 2) = 1e-6;     % Termination tolerance for X.(Default: 1e-4).
  OPTIONS( 3) = 1e-6;     % Termination tolerance on F.(Default: 1e-4).
% OPTIONS( 5) = 1;        % Algorithm: Strategy:  1 = Gauss-Newton, 0 = Levenberg-Marquardt
% OPTIONS( 7) =           % Algorithm: Line Search Algorithm. (Default 0)
% OPTIONS( 8) =           % Function value. (Lambda in goal attainment. )
% OPTIONS( 9) =           % Set to 1 if you want to check user-supplied gradients
% OPTIONS(10) =           % Number of Function and Constraint Evaluations.
% OPTIONS(11) =           % Number of Function Gradient Evaluations.
% OPTIONS(14) =  100;     % Maximum number of iterations. (Default 100*no. of variables)
% OPTIONS(15) =           % Used in goal attainment for special objectives. 
% OPTIONS(16) = 1e-8;     % Minimum change in variables for finite difference gradients.
% OPTIONS(17) = 1e-5;     % Maximum change in variables for finite difference gradients.
% OPTIONS(18) = .1;       % Step length. (Default 1 or less). 
  

%%
 % Estimation from lines.
 %%
if strcmp(method,'lines')
  L = L_uw;

  % Remove points not used in any line
  [ud,L] = onlyused(ud,L);

  % Call gradient descent
  Mask = [1 1 0 ones(1,length(p0)-3)]';
  p = lstsqm( 'objln', p0, Mask, OPTIONS, [], ud, L, SWAP_DISTFCN );

%%
 % Estimation from homography.
 %%
elseif strcmp(method,'homo')
  uw = L_uw;

  %%
   % Find initial estimate of homography between uw and ud.
   % This estimate is found as such a homography H that
   %   |p2e(H*e2p(uw))-ud| ---> min
   %%
  H0 = uu2homog(uw,ud,'norm');

  % Call gradient descent
  Mask = [ones(1,9) 1 1 0 ones(1,length(p0)-3)]';
  Hp = lstsqm( 'objho', [H0(:); p0], Mask, OPTIONS, [], ud, uw, SWAP_DISTFCN );

  % extract H, p from estimated Hp
  H = reshape(Hp(1:9),3,3);
  p = Hp(10:length(Hp));

  % unnormalize H
  H = inv(Aud)*H;

end

%%
 % unnormalize p
 %%
p = psimi2p(p,inv(Aud));

return
