function simul
%simul Testing the correction of radial lens distortion on synthetic data.
%
%   THE TEST ALGORITHM
%
% The test consists of the following steps:
%
%   1. Generating world points uw.
%      The points in the 3D world are generated. These points are coplanar and their
%      coordinates are expressed in the system of the plane they lie in, eg, their
%      coordinates are 2D. The points form a regular grid. The numbers of grid 
%      horizontal and vertical lines are given by GrSize. The range of grid point 
%      coordinates is always <0,1>x<0,1>.
%
%   2. Generating linear-camera image points ul.
%      Projection of the world points by linear camera is done by applying 
%      a homography to the world points uw. The homography is such that the resulting
%      image size is approx. ImSize.
%
%   3. Applying radial lens distortion to ul, the result is distorted points ud.
%      See parameters p and the help for U2UDIST for the distortion model.
%
%   4. Adding noise to ud, the result is the points un.
%      The noise portion is given by Sigma. Uniform noise is currently used.
%
%   5. Estimating the lens distortion parameters pu using the points ud.
%      Using RDESTIM, distortion parameters p are estimated.
%      The initial estimate of p is p0.
%
%   6. Undistorting the distorted points ud by pu, the result is uu.
%      The undistortion function UDIST2U is applied to the NOISELESS distorted points.
%
%   Steps 4, 5, 6 are repeated for various portions of noise Sigma and various trials.
%   The results are visualized.
%

%       Author       : (c) T. Werner and T. Pajdla, {werner,pajdla}@cmp.felk.cvut.cz
%                      07/09/98
%                      Center for Machine Perception, Czech Technical University, Prague
%       Documentation: ftp://cmp.felk.cvut.cz/pub/cmp/articles/werner/tr97-138.ps.gz     
%       Language     : Matlab 5.1, (c) MathWorks                         
%       Last change  : $Id: simul.m,v 1.1 1998/09/07 15:54:15 Matlab Exp $
%       Status       : Ready ($Source: /home.others/Matlab/Matlab/radial/RCS/simul.m,v $)


rand('seed',0);

%if isempty(findobj('Type','figure')),
%  close all
%  loaddsk simul
%end


%%
 % Parameters of the experiment.
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Colors in the figures.
COLOR_lin     = 'g';  % image points captured by linear camera
COLOR_distort = 'm';  % linear camera points distorted by radial distortion
COLOR_noise   = 'w';  % distorted points with added noise
COLOR_undist  = 'r';  % undistorted points by the estimated distortion parameters
COLOR_res     = 'y';  % color of plot of residuals

GrSize = [7 7]';  % grid size
ImSize = [800  600]';  % image size
%p      = [.4 .4   0  0.02  ]';  % lens radial distortion parameters, see U2UDIST, p = [u0; r0; k]
%p0     = [.3 .6   0  0     ]';  % initial estimate of p 
%pMask  = [1  1    0     1     ]';  % only parameters with 1's will change during optimization
p      = [400 400   0  0.05e-6 -0.08e-12 ]';  % lens radial distortion parameters, see U2UDIST, p = [u0; r0; k]
p0     = [500 300   0  0     0    ]';  % initial estimate of p 
pMask  = [1  1    0  1     1    ]';  % only parameters with 1's will change during optimization
Ntrials = 1;
Sigma = [0:.2:10];
FROM_HOMO    = 1;  % 1 = estimate from homography, 0 = estimate from lines
SWAP_DISTFCN = 0;  % 1 = swap direct/inverse distortion polynom, 0 = do not swap


%%
 % Create scene points. The points are coplanar and form a regular grid.
 % All grid points coords fit in interval <0,1>, their number is given by GrSize.
 % The points are expressed in 2D coords in the grid system, their
 % 3D coords are not explicitely available.
 %   uw ... the points in the World
 %   L ... indices of grid horizontal and vertical lines (see RDESTIM)
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% create the points uw
gax = linspace(0,1,GrSize(1));
gay = linspace(0,1,GrSize(2));  % coords of intersections on grid axes
uwx = ones(GrSize(2),1) * gax;
uwy = gay' * ones(1,GrSize(1)); % x- and y-coords of the grid points (grid lattice)
uw = [uwx(:)'; uwy(:)'];        % grid lattice linearized

% create L
iL = []; jL = []; vL = []; ni = 0;
for xi = gax,
  ni = ni + 1;
  i = find(uw(1,:)==xi);
  iL = [iL 1:length(i)]; 
  jL = [jL ni*ones(1,length(i))];
  vL = [vL i];
end
for yi = gay,
  ni = ni + 1;
  i = find(uw(2,:)==yi);
  iL = [iL 1:length(i)]; 
  jL = [jL ni*ones(1,length(i))];
  vL = [vL i];
end
L = sparse(iL,jL,vL);
clear iL jL vL ni xi yi gay gax uwx uwy


%%
 % Apply a homography to scene points uw to create a linear camera image.
 % ul ... image points captured by Linear camera
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
H1 = [ImSize(1) 0 0
      0 ImSize(2) 0
      0 0         1];        % magnify to ImSize
H2 = eye(3) + 0.2*rand(3);   % slightly tilt
ul = p2e(H1*H2*e2p(uw));


%%
 % Apply radial lens distortion to linear image points ul.
 % p ... lens distortion parameters, see above
 % ud ... Distorted image points.
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
ud = u2udist(p,ul,SWAP_DISTFCN);


%%
 % The loop for performing undistortion for various noise added
 % to points ud and various trials for the noise of the same sigma.
 % 
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Sigma_pu = []; Sigma_noi = []; Sigma_res = []; Sigma_err = [];
for sigma = Sigma
  disp(['**** Sigma of noise = ' num2str(sigma)]);
  Trial_pu = []; Trial_noi = []; Trial_res = []; Trial_err = [];

  for trial = 1:Ntrials
    fprintf('******** Trial #%i\n',trial);
    figs hide

    %%
     % Add noise to distorted image points ud. The resulting points un
     % are in fact what we measure in the image.
     % un ... distorted image points with added Noise
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    un = ud + sigma*(2*rand(size(ud))-1);  % uniform noise
    un = ud + sigma*randn(size(ud));  % normal noise
    noi = abs(un(:)-ud(:));

save

    %%
     % Estimate radial lens distortion parameters from un.
     % pu ... estimated distortion parameters
     % Hu ... estimated homography, see RDESTIM
     % (Ideally it should be pu==p.)
     %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if FROM_HOMO
      [pu,Hu] = rdestim('homo',p0,un,uw,SWAP_DISTFCN);
      res = objho([Hu(:); pu],ud,uw,SWAP_DISTFCN);
    else
      pu = rdestim('lines',p0,un,L,SWAP_DISTFCN);
      res = objln(pu,ud,L,SWAP_DISTFCN);        
    end

    %%
     % Undistort ud (NOT un!) using pu.
     % uu ... undistorted points
     %%
    uu = udist2u(pu,ud,SWAP_DISTFCN);
    err = abs(ul(:)-uu(:));

    %%
     % Store pu, res, err
     %%
    Trial_pu  = [Trial_pu  pu ];    
    Trial_noi = [Trial_noi mean(noi)];
    Trial_res = [Trial_res mean(res)];    
    Trial_err = [Trial_err mean(err)];    

    %%
     % Plot cartesian coords of the involved points
     %%
    figure(1), set(gcf,'name',['ul=' COLOR_lin ',ud=' COLOR_distort ',un=' COLOR_noise ',uu=' COLOR_undist]);
    plot( ul(1,:), ul(2,:), [COLOR_lin     'x'],...
          ud(1,:), ud(2,:), [COLOR_distort 'x'],...
          un(1,:), un(2,:), [COLOR_noise   'x'],...
          uu(1,:), uu(2,:), [COLOR_undist  'x'] );

    %%
     % Plot histograms of differences of the involved points.
     %%
    figure(2), set(gcf,'name','Distort=|ud-ul|'), [hn,hx]=hist(abs(ud(:)-ul(:)),10); bar(hx,hn,COLOR_distort);
    figure(3), set(gcf,'name','Noise=|un-ud|'  ), [hn,hx]=hist(abs(un(:)-ud(:)),10); bar(hx,hn,COLOR_noise  );
    figure(4), set(gcf,'name','Error=|uu-ul|'  ), [hn,hx]=hist(abs(uu(:)-ul(:)),10); bar(hx,hn,COLOR_undist );

    %%
     % Plot differences of radii of the involved points.
     % The radii are computed from the true projection center u0.
     %%
    u0long = p(1:2)*ones(1,size(ul,2));
    rl = sqrt( sum((ul-u0long).^2) );  [rl,rli] = sort(rl);
    rd = sqrt( sum((ud-u0long).^2) );  rd = rd(:,rli);
    rn = sqrt( sum((un-u0long).^2) );  rn = rn(:,rli);
    ru = sqrt( sum((uu-u0long).^2) );  ru = ru(:,rli);
    clear u0long
    figure(8), set(gcf,'name',['rd-rl=' COLOR_distort ',rn-rl=' COLOR_noise ',ru-rl=' COLOR_undist])  
    plot( rl, rd-rl, COLOR_distort,...
          rl, rn-rl, COLOR_noise,...
          rl, ru-rl, COLOR_undist );

    % Plot original and estimated distortion polynom
    rl  = linspace( 0, sqrt(p(1:2)'*p(1:2)), 1000 );
    figure(9), set(gcf,'name',['Orig dr = ' COLOR_distort ', estim dr = ' COLOR_undist])
    plotdr(p,rl,COLOR_distort); hold on
    plotdr(pu,rl,COLOR_undist); hold off

    disp(['Mean residuals = ' num2str(mean(res))]);
    pu
    drawnow

    figs unhide
  end
  fprintf('\n');

  Sigma_pu  = [Sigma_pu  Trial_pu];
  Sigma_noi = [Sigma_noi mean(Trial_noi)];
  Sigma_res = [Sigma_res mean(Trial_res)];
  Sigma_err = [Sigma_err mean(Trial_err)];

  %%
   % Plot results for the current sigma.
   %%
  figure(7), set(gcf,'name','Noise=mean( |un-ud| )'), plot( Sigma_noi, COLOR_noise  )
  figure(5), set(gcf,'name','Mean fit residuals'   ), plot( Sigma_res, COLOR_res )
  figure(6), set(gcf,'name','Error=mean( |uu-ul| )'), plot( Sigma_err, COLOR_undist  )

  pause
end

return


% demonstration that by changing r0 and k we can achieve arbitrarily small
% value of objln
r = 0.5; u0 = [.5 .5]'; r0 = 1; k = .1; R = [];
a=1:-.1:.1; ka = a*k; r0a = sqrt( (1-a.*(1-k*r0^2))./(a.*k) );
for aa=1:length(a), 
  R(aa) = sum(objln([u0; r0a(aa); ka(aa)],un,L,SWAP_DISTFCN));
end
